package social.agents;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import repast.simphony.context.Context;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.ContextUtils;
import social.bbdd.BbddManager;
import social.content.IComplaint;
import social.content.IContent;
import social.content.complaint.Complaint;
import social.context.ContextData;
import social.distribution.Gamma;
import social.agents.profile.ComplaintProfile;
import social.agents.profile.UploadProfile;
import social.agents.profile.ViewProfile;

/**
 * Basic agent
 * @author davidsanchezpinsach
 *
 */
public class Agent {
	private ContinuousSpace<Object> space;
	private Grid<Object> grid;
	
	//The date when the user creates the account
	protected Calendar dataUserCreation;
	
	//Position of the agent in the grid and comments
	protected int posAgentRow;
	
	//Database connection
	protected BbddManager bbddManager;
	
	//Profiles of the agent
	protected UploadProfile upLoadProfile;
	protected ViewProfile viewProfile;
	protected ComplaintProfile complaintProfile;
	protected Context<Object> context;
	
	//Sections
	private final int sectionTheReporter = 1;
	private final int sectionForum = 2;
	private final int sectionPhotoVideo = 3;
	
	//Type of view profile
	private final int contentByOrder = 0;
	private final int contentByMostViews = 1;
	private final int contentByRandom = 2;
	
	//Agents characteristics
	private int type;
	private int quantity;
	private String name, population;
		
	/**
	 * Constructor of the agent class with space and grid
	 * @param space Space of the context
	 * @param grid Grid of the context
	 * @param posAgentRow Position of the agent
	 * @param upLoadProfile UploadProfile
	 * @param viewProfile ViewProfile
	 * @param complaintProfile Complaint profile
	 */
	public Agent(ContinuousSpace<Object> space , Grid<Object> grid, int posAgentRow, UploadProfile upLoadProfile, ViewProfile viewProfile, ComplaintProfile complaintProfile){
		this.space = space;
		this.grid = grid;
		this.posAgentRow = posAgentRow;
		
		this.upLoadProfile = upLoadProfile;
		this.viewProfile = viewProfile;
		this.complaintProfile = complaintProfile;

		dataUserCreation = Calendar.getInstance();
		
		//Create the class manager to the connections between bbdd and repast
		bbddManager = new BbddManager();
	}
	
	/**
	 * Constructor to generate poblations of the agents with a same type
	 * @param type Type of the agent
	 * @param quantity Quantity of the agent
	 * @param name Name of the agent
	 * @param up UploadProfile
	 * @param vp ViewProfile
	 * @param cp ComplaintProfile
	 * @param population Name of the population to use
	 */
	public Agent(int type, int quantity, String name, UploadProfile up, ViewProfile vp, ComplaintProfile cp, String population){
		this.type = type;
		this.quantity = quantity;
		this.name = name;
		this.upLoadProfile = up;
		this.viewProfile = vp;
		this.complaintProfile = cp;
		this.population = population;
	}
	
	/**
	 * Method to generate uploadContent and put this in the context
	 */
	public void upLoadContent(){
		ContextData contextData = (ContextData) grid.getObjectAt(0,0);
		int num = contextData.nextIntRandom(100);
		if(num <= upLoadProfile.getUpLoadContent()*100){
			IContent comment = upLoadProfile.getFirstListUploadProfile();
			int positionComment = 0;
			if(comment != null){
				context = ContextUtils.getContext(this);
				context.add(comment);
	
				switch(comment.getSection()){
					case sectionTheReporter:
						if(upLoadProfile.getNumCommentsTheReporter() < upLoadProfile.getMaxCommentsTheReporter()){
							upLoadProfile.setNumCommentsTheReporter(upLoadProfile.getNumCommentsTheReporter()+1);
							positionComment = upLoadProfile.getNumCommentsTheReporter();
							contextData.addContentListSectionTheReporter(comment);
							contextData.addContentMostViewTheReporter(comment);
						}else{
							comment = null;
						}
						break;
					case sectionForum:
						if(upLoadProfile.getNumCommentsForum() < upLoadProfile.getMaxCommentsForum()){
							upLoadProfile.setNumCommentsForum(upLoadProfile.getNumCommentsForum()+1);
							positionComment = contextData.getMaxCommentsTheReporter()+upLoadProfile.getNumCommentsForum()+1;
							contextData.addContentListSectionForum(comment);
							contextData.addContentMostViewForum(comment);
						}else{
							comment = null;
						}
						break;
					case sectionPhotoVideo:
						if(upLoadProfile.getNumCommentsPhotoVideo() < upLoadProfile.getMaxCommentsPhotoVideo()){
							upLoadProfile.setNumCommentsPhotoVideo(upLoadProfile.getNumCommentsPhotoVideo()+1);
							positionComment = contextData.getMaxCommentsTheReporter()+contextData.getMaxCommentsForum()+upLoadProfile.getNumCommentsPhotoVideo()+2;
							contextData.addContentListSectionPhotoVideo(comment);
							contextData.addContentMostViewPhotoVideo(comment);
						}else{
							comment = null;
						}
						break;
				}
				if(comment!=null){
					space.moveTo(comment, positionComment, posAgentRow); 
					grid.moveTo(comment, positionComment, posAgentRow);
				}
			}
		}
	}
	
	/**
	 * Method to view content of the context
	 * @return The content that was visit
	 */
	public IContent viewContent(){
		int num = 0, size = 0, numGamma;
		IContent content = null;
		ContextData contextData = (ContextData) grid.getObjectAt(0,0);
		num = contextData.nextIntRandom(100);
		double theReporter = viewProfile.getTheReporter()*100;
		double forum = viewProfile.getForum()*100;
		Gamma g = new Gamma();	//Create distribution gamma
		switch(viewProfile.getViewMode()){
			case contentByOrder:
				if((num < theReporter) && (!contextData.getListSectionTheReporter().isEmpty())){
					numGamma = g.getNum(contextData.getListSectionTheReporter());
					contextData.getContentListSectionTheReporter(numGamma).setNumViews(contextData.getContentListSectionTheReporter(numGamma).getNumViews()+1);
					content = contextData.getContentListSectionTheReporter(numGamma);
				}else if((num > theReporter && num <= theReporter+forum) && (!contextData.getListSectionForum().isEmpty())){	
					numGamma = g.getNum(contextData.getListSectionForum());
					contextData.getContentListSectionForum(numGamma).setNumViews(contextData.getContentListSectionForum(numGamma).getNumViews()+1);
					content = contextData.getContentListSectionForum(numGamma);
				}else if(!contextData.getListSectionPhotoVideo().isEmpty() && num != 0){
					numGamma = g.getNum(contextData.getListSectionPhotoVideo());
					contextData.getContentListSectionPhotoVideo(numGamma).setNumViews(contextData.getContentListSectionPhotoVideo(numGamma).getNumViews()+1);
					content = contextData.getContentListSectionPhotoVideo(numGamma);
				}
				break;
			case contentByMostViews:
				if((num <= theReporter) && (!contextData.getMostViewListTheReporter().isEmpty())){
					numGamma = g.getNum(contextData.getMostViewListTheReporter());
					contextData.upDateView(contextData.getMostViewListTheReporter(), numGamma);
					content = contextData.getContentMostViewTheReporter(numGamma);
				}else if((num > theReporter && num <= theReporter+forum) && (!contextData.getMostViewListForum().isEmpty())){			
					numGamma = g.getNum(contextData.getMostViewListForum());
					contextData.upDateView(contextData.getMostViewListForum(), numGamma);
					content = contextData.getContentMostViewForum(numGamma);
				}else if(!contextData.getMostViewListPhotoVideo().isEmpty() && num != 0){
					numGamma = g.getNum(contextData.getMostViewListPhotoVideo());
					contextData.upDateView(contextData.getMostViewListPhotoVideo(), numGamma);
					content = contextData.getContentMostViewPhotoVideo(numGamma);
				}
				break;
			case contentByRandom:
				if((num < theReporter) && (!contextData.getListSectionTheReporter().isEmpty())){
					size = contextData.getListSectionTheReporter().size()-1;
					if(size != 0){
						size = contextData.nextIntRandom(size); //Give random item
					}
					contextData.getContentListSectionTheReporter(size).setNumViews(contextData.getContentListSectionTheReporter(size).getNumViews()+1);
					content = contextData.getContentListSectionTheReporter(size);
				}else if((num > theReporter && num <= theReporter+forum) && (!contextData.getListSectionForum().isEmpty())){	
					size = contextData.getListSectionTheReporter().size()-1;
					if(size != 0){
						size = contextData.nextIntRandom(size); //Give random item
					}
					contextData.getContentListSectionForum(size).setNumViews(contextData.getContentListSectionForum(size).getNumViews()+1);
					content = contextData.getContentListSectionForum(size);
				}else if((!(contextData.getListSectionPhotoVideo().isEmpty())) && (num != 0)){
					size = contextData.getListSectionPhotoVideo().size()-1;
					if(size != 0){
						size = contextData.nextIntRandom(size); //Give random item
						
					}
					contextData.getContentListSectionPhotoVideo(size).setNumViews(contextData.getContentListSectionPhotoVideo(size).getNumViews()+1);
					content = contextData.getContentListSectionPhotoVideo(size);
				}
				break;
		}
		return content;
	}
	
	/**
	 * Method to complaint contents
	 * @param content Content that the agent visited
	 */
	public void complaintContent(IContent content){
		int x, y, num;
		double complaintType = 0;
		ContextData contextData = (ContextData) grid.getObjectAt(0,0);
		num = contextData.nextIntRandom(100);
		if(content!=null){
			//Get the position of the content and see if it already has complain 
			GridPoint gp = grid.getLocation(content);
			x = gp.getX();
			y = gp.getY();
			context = ContextUtils.getContext(this);
			IComplaint complaint = (IComplaint)grid.getObjectAt(x,y-1);
			
			//Check that the comment isn't of this agent
			if(y!=posAgentRow){
				switch(content.getContentType()){
					//Spam
					case 2:
						complaintType = complaintProfile.getSpam()*100;
						break;
					//Porn
					case 3:
						complaintType = complaintProfile.getPorn()*100;
						break;
					//Violent
					case 4:
						complaintType = complaintProfile.getViolent()*100;
						break;
					//Wrong placement
					case 5:
						complaintType = complaintProfile.getWrongPlacement()*100;
						break;
					//Insult
					case 6:
						complaintType = complaintProfile.getInsult()*100;
						break;
				}
				if((num<=complaintType)&&(complaintType!=0)){
					if(complaint != null){
						complaint.setNumComplaints(complaint.getNumComplaints()+1);
					}else{
						complaint = new Complaint(content.getContentType(),1);
						context.add(complaint);
						space.moveTo(complaint, x, y-1); 
						grid.moveTo(complaint, x, y-1);
					}
				}
			}
		}
	}
	
	/**
	 * Method to view and complaint content
	 */
	public void viewAndComplaintContent(){
		//Firt view some upload content
		IContent content = viewContent();
		//Complaint or not complaint the content visited
		complaintContent(content);
	}
	
	/**
	 * Metod to show the data of the user creation
	 */
	public void showDataUserCreation(){
		DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
		System.out.println(dateFormat.format(dataUserCreation.getTime()));
	}
	
	public int getType() {
		return type;
	}

	public void setType(int type) {
		this.type = type;
	}
	
	public int getQuantity() {
		return quantity;
	}

	public void setQuantity(int quantity) {
		this.quantity = quantity;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public UploadProfile getUp() {
		return upLoadProfile;
	}

	public void setUp(UploadProfile upLoadProfile) {
		this.upLoadProfile = upLoadProfile;
	}

	public ViewProfile getVp() {
		return viewProfile;
	}

	public void setVp(ViewProfile viewProfile) {
		this.viewProfile = viewProfile;
	}

	public ComplaintProfile getCp() {
		return complaintProfile;
	}

	public void setCp(ComplaintProfile complaintProfile) {
		this.complaintProfile = complaintProfile;
	}
	
	public String getPopulation(){
		return population;
	}

}
