package social.agents.profile;

import java.util.ArrayList;
import repast.simphony.space.grid.Grid;
import social.bbdd.BbddManager;
import social.content.IContent;
import social.context.ContextData;

/**
 * Upload profile of the agent
 * @author davidsanchezpinsach
 *
 */
public class UploadProfile{
	
	//Type of different upload content percentage
	private double correct = 0;
	private double wrongPlacement = 0;
	private double insult = 0;
	private double spam = 0;
	private double violent = 0;
	private double porn = 0;
	
	//Percentage to upload of the content.
	private double upLoadContent = 0;
	
	private int maxCommentsPhotoVideo;
	private int maxCommentsForum;
	private int maxCommentsTheReporter;
	
	public final int sectionTheReporter = 1;
	public final int sectionForum = 2;
	public final int sectionPhotoVideo = 3;
	
	//Number of specific section
	private int numCommentsPhotoVideo = 0;
	private int numCommentsForum = 0;
	private int numCommentsTheReporter = 0;
	
	//Type of comments
	private final int idOk=1;
	private final int idSpam=2;
	private final int idPorn=3;
	private final int idViolent=4;
	private final int idWrongPlacement=5;
	private final int idInsult=6;
	
	//List of upload profile
	protected ArrayList<IContent> listUploadProfile;
	
	//Total of comments of the uploadProfile
	private int totalComments = 0;

	/**
	 * Constructor of the uploadProfile
	 * @param correct Correct content frequency
	 * @param wrongPlacement WrongPlacement content frequency
	 * @param insult Insult content frequency
	 * @param spam Spam content frequency
	 * @param violent Ciolent content frequency
	 * @param porn Porn content frequency
	 * @param upLoadContent UpLoadContent content frequency
	 */
	public UploadProfile(double correct, double wrongPlacement, double insult, double spam, double violent, double porn, double upLoadContent){
		this.correct = correct;
		this.wrongPlacement = wrongPlacement;
		this.insult = insult;
		this.spam = spam;
		this.violent = violent;
		this.porn = porn;
		this.upLoadContent = upLoadContent;
		
		listUploadProfile = new ArrayList<IContent>();
	}

	public double getCorrect() {
		return correct;
	}

	public void setCorrect(double correct) {
		this.correct = correct;
	}

	public double getWrongPlacement() {
		return wrongPlacement;
	}

	public void setWrongPlacement(double wrongPlacement) {
		this.wrongPlacement = wrongPlacement;
	}

	public double getInsult() {
		return insult;
	}

	public void setInsult(double insult) {
		this.insult = insult;
	}

	public double getSpam() {
		return spam;
	}

	public void setSpam(double spam) {
		this.spam = spam;
	}

	public double getViolent() {
		return violent;
	}

	public void setViolent(double violent) {
		this.violent = violent;
	}

	public double getPorn() {
		return porn;
	}

	public void setPorn(double porn) {
		this.porn = porn;
	}
	
	public ArrayList<IContent> getListUploadProfile() {
		return listUploadProfile;
	}

	public void setListUploadProfile(ArrayList<IContent> listUploadProfile) {
		this.listUploadProfile = listUploadProfile;
	}
	
	public IContent getFirstListUploadProfile(){
		IContent content = null;
		if(listUploadProfile.size()>0){
			content = listUploadProfile.remove(0);
		}
		return content;	
	}
	
	/**
	 * Method to generate random uploadlist from BBDD
	 * @param bbddManager Class to manager BBDD
	 * @param grid Grid of the context
	 */
	public void generateRandomUploadList(BbddManager bbddManager, Grid<Object> grid){
		int index;
		int numCorrect = (int)(totalComments*correct);
		int numWrongPlacement = (int)(totalComments*wrongPlacement);
		int numInsult = (int)(totalComments*insult);
		int numSpam = (int)(totalComments*spam);
		int numViolent = (int)(totalComments*violent);
		int numPorn = (int)(totalComments*porn);
		
		ContextData contextData = (ContextData) grid.getObjectAt(0,0);
		
			
		//Generate the random list of the sections
		int sectionSize = totalComments/3;
		ArrayList<Integer> sectionList = new ArrayList<Integer>();
		for(int i=0; i<sectionSize; i++){
			sectionList.add(sectionPhotoVideo);
		}
		
		for(int i=0; i<sectionSize; i++){
			sectionList.add(sectionForum);
		}
		
		for(int i=0; i<sectionSize; i++){
			sectionList.add(sectionTheReporter);
		}
		
		int num = sectionSize*3;
		//Random the elements of the list
		ArrayList<Integer> tempListSection = new ArrayList<Integer>();
		for(int i=0; i<num; i++){
			index = contextData.nextIntRandom(sectionList.size());
			tempListSection.add(sectionList.remove(index));
		}
		sectionList = tempListSection;
		
		int format = 0;//text  1//photo 2//video
		
		for(int i=0; i<numCorrect&&!sectionList.isEmpty(); i++){
			IContent comment = bbddManager.sendQuery(idOk, format, sectionList.remove(0));
			if(comment != null){
				listUploadProfile.add(comment);
			}
		}
		for(int i=0; i<numWrongPlacement&&!sectionList.isEmpty(); i++){
			IContent comment = bbddManager.sendQuery(idWrongPlacement, format, sectionList.remove(0));
			if(comment != null){
				listUploadProfile.add(comment);
			}
		}
		
		for(int i=0; i<numInsult&&!sectionList.isEmpty(); i++){
			IContent comment = bbddManager.sendQuery(idInsult, format, sectionList.remove(0));
			if(comment != null){
				listUploadProfile.add(comment);
			}
		}
		
		for(int i=0; i<numSpam&&!sectionList.isEmpty(); i++){
			IContent comment = bbddManager.sendQuery(idSpam, format, sectionList.remove(0));
			if(comment != null){
				listUploadProfile.add(comment);
			}
		}
		
		for(int i=0; i<numViolent&&!sectionList.isEmpty(); i++){
			IContent comment = bbddManager.sendQuery(idViolent, format, sectionList.remove(0));
			if(comment != null){
				listUploadProfile.add(comment);
			}
		}
		for(int i=0; i<numPorn&&!sectionList.isEmpty(); i++){
			IContent comment = bbddManager.sendQuery(idPorn, format, sectionList.remove(0));
			if(comment != null){
				listUploadProfile.add(comment);
			}
		}
		
		//Random the elements of the list
		ArrayList<IContent> tempListComment = new ArrayList<IContent>();
		for(int i=0; i<totalComments; i++){
			if(listUploadProfile.size()!=0){
				index = contextData.nextIntRandom(listUploadProfile.size());
				tempListComment.add(listUploadProfile.remove(index));
			}
		}
		listUploadProfile = tempListComment;
	}
	
	public int getMaxCommentsPhotoVideo() {
		return maxCommentsPhotoVideo;
	}

	public void setMaxCommentsPhotoVideo(int maxCommentsPhotoVideo) {
		this.maxCommentsPhotoVideo = maxCommentsPhotoVideo;
	}

	public int getMaxCommentsForum() {
		return maxCommentsForum;
	}

	public void setMaxCommentsForum(int maxCommentsForum) {
		this.maxCommentsForum = maxCommentsForum;
	}

	public int getMaxCommentsTheReporter() {
		return maxCommentsTheReporter;
	}

	public void setMaxCommentsTheReporter(int maxCommentsTheReporter) {
		this.maxCommentsTheReporter = maxCommentsTheReporter;
	}

	public int getTotalComments() {
		return totalComments;
	}

	public void setTotalComments(int totalComments) {
		this.totalComments = totalComments;
	}

	public double getUpLoadContent() {
		return upLoadContent;
	}

	public void setUpLoadContent(double upLoadContent) {
		this.upLoadContent = upLoadContent;
	}

	public int getNumCommentsPhotoVideo() {
		return numCommentsPhotoVideo;
	}

	public void setNumCommentsPhotoVideo(int numCommentsPhotoVideo) {
		this.numCommentsPhotoVideo = numCommentsPhotoVideo;
	}

	public int getNumCommentsForum() {
		return numCommentsForum;
	}

	public void setNumCommentsForum(int numCommentsForum) {
		this.numCommentsForum = numCommentsForum;
	}

	public int getNumCommentsTheReporter() {
		return numCommentsTheReporter;
	}

	public void setNumCommentsTheReporter(int numCommentsTheReporter) {
		this.numCommentsTheReporter = numCommentsTheReporter;
	}
}
