package social.bbdd;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Random;

import social.agents.Agent;
import social.content.IContent;
import social.content.comment.CorrectComment;
import social.content.comment.InsultComment;
import social.content.comment.PornComment;
import social.content.comment.SpamComment;
import social.content.comment.ViolentComment;
import social.content.comment.WrongPlacementComment;
import social.context.ContextData;
import social.agents.profile.ComplaintProfile;
import social.agents.profile.UploadProfile;
import social.agents.profile.ViewProfile;

/**
 * Class to manager BBDD
 * @author davidsanchezpinsach
 *
 */
public class BbddManager {

	Connection connection; 
	
	//Format of the content
	private final int txt = 0;
	private final int img = 1;
	private final int video = 2;
	
	//Type of comments
	protected final int idOk=1;
	protected final int idSpam=2;
	protected final int idPorn=3;
	protected final int idViolent=4;
	protected final int idWrongPlacement=5;
	protected final int idInsult=6;
	
	/**
	 * Method to established connection to BBDD
	 * @param agent Agent
	 */
	public void establishedConnection(String agent){
         String userName = "root";
         String password = "";
         String direction = "jdbc:mysql://localhost/virtualCommunities";
         try {
			Class.forName ("com.mysql.jdbc.Driver").newInstance();
			connection = DriverManager.getConnection(direction, userName, password);
			System.out.println ("\nDatabase connection established "+agent+ ": "+Calendar.getInstance().getTime());
		} catch (Exception e) {
			 System.err.println ("Cannot connect to database server"+agent+ ": "+Calendar.getInstance().getTime());
		}
	}
	
	/**
	 * Method to finish the connection to BBDD
	 * @param agent
	 */
	public void terminatedConnection(String agent){
		if (connection != null){
           try{
        	   connection.close();
               System.out.println ("Database connection terminated "+agent+ ": "+Calendar.getInstance().getTime());
           }catch (Exception e) { /* ignore close errors */ }
       }
	}

	/**
	 * Method send query and give the datas
	 * @param idType Identificator of the type of the content
	 * @param format Format of the content
	 * @param idSection Identificator of the section
	 * @return
	 */
	public IContent sendQuery(int idType, int format, int idSection){
		IContent comment = null;
		String stringFormat="";
		switch(format){
			case 0:
				stringFormat = "Text";
				break;
			case 1:
				stringFormat = "Image";
				break;
			case 2:
				stringFormat = "Video";
				break;
		}
		try {
        	PreparedStatement statement = connection.prepareStatement("SELECT * FROM content WHERE content_type="+idType+" AND section_type="+idSection+" AND file='"+stringFormat+"'");
			ResultSet result = statement.executeQuery();
			if(result.next()){
				int id = result.getInt("id");
				int content_type = result.getInt("content_type");
				String file = result.getString("file");
				String url = result.getString("url");
				String message = result.getString("message");
				int section = result.getInt("section_type");
				switch(format){
				case txt:
					switch(idType){
						case idOk:
							comment = new CorrectComment(id, content_type, file, url, message, section);
							break;
						case idSpam:
							comment = new SpamComment(id, content_type, file, url, message, section);
							break;
						case idPorn:
							comment = new PornComment(id, content_type, file, url, message, section);
							break;
						case idViolent:
							comment = new ViolentComment(id, content_type, file, url, message,section);
							break;		
						case idWrongPlacement:
							comment = new WrongPlacementComment(id, content_type, file, url, message, section);
							break;	
						case idInsult:
							comment = new InsultComment(id, content_type, file, url, message, section);
							break;	
					}
					break;
				case img:
					break;
				case video:
					break;
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("Error with SQL query. The format isn't correct, see the manual of Mysql");
			return null;
		}
		return comment;
	}
	
	public Connection getConn() {
		return connection;
	}

	public void setConn(Connection connection) {
		this.connection = connection;
	}

	/**
	 * Method to get popupaltion datas from BBDD
	 * @return
	 */
	public ArrayList<String> getPopulations() {
		ArrayList<String> popuName = new ArrayList<String>();
		
		try{
			PreparedStatement statement = connection.prepareStatement("SELECT * FROM agent_population");
			ResultSet result = statement.executeQuery();
			while(result.next()){
				popuName.add(result.getString("populationName"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("Error with SQL query. The format isn't correct, see the manual of Mysql");
		}
		return popuName;
	}

	public ArrayList<String> getAgentTypes() {
		ArrayList<String> agentTypes = new ArrayList<String>();
		try{
			PreparedStatement statement = connection.prepareStatement("SELECT * FROM agent_type");
			ResultSet result = statement.executeQuery();
			while(result.next()){
				agentTypes.add(result.getString("typeName"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("Error with SQL query. The format isn't correct, see the manual of Mysql");
		}
		
		return agentTypes;
	}

	/**
	 * Method to gets the agents profile
	 * @param contextData Context data of simulation
	 * @param uploadProfile Upload profile of the agent
	 * @param viewProfile View profile of the agent
	 * @param complaintProfile Complaint profile of the agent
	 * @param randomUpload Random
	 * @param population The name of population
	 * @return Return arraylist of the agents
	 */
	public ArrayList<Agent> getAgentsProfile(ContextData contextData, UploadProfile uploadProfile, ViewProfile viewProfile, ComplaintProfile complaintProfile, Random randomUpload, String population) {
		ArrayList<Agent> agente = new ArrayList<Agent>();
		int id_agent_population = 0;
		ArrayList<Integer> id_agent_type = new ArrayList<Integer>();
		ArrayList<Integer> id_agent_profile = new ArrayList<Integer>();
		ArrayList<Integer> quantity = new ArrayList<Integer>();

		try{
			PreparedStatement statement = connection.prepareStatement("SELECT id_agent_population FROM agent_population WHERE populationName = '"+population+"'");
			ResultSet result = statement.executeQuery();
			while(result.next()){
				id_agent_population = result.getInt("id_agent_population");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("Error with SQL query. The format isn't correct, see the manual of Mysql");
		}
		
		try{
			PreparedStatement statement = connection.prepareStatement("SELECT * FROM population WHERE id_population = "+id_agent_population);
			ResultSet result = statement.executeQuery();
			while(result.next()){
				id_agent_type.add(result.getInt("id_agent_type"));
				id_agent_profile.add(result.getInt("id_agent_profile"));
				quantity.add(result.getInt("quantity"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("Error with SQL query. The format isn't correct, see the manual of Mysql");
		}
		
		for(int i = 0 ; i < id_agent_profile.size() ; i++){
			try{
				PreparedStatement statement = connection.prepareStatement("SELECT * FROM agent_profile WHERE id_profile = "+id_agent_profile.get(i));
				ResultSet result = statement.executeQuery();
				while(result.next()){
					
					//Upload profile settings
					uploadProfile = new UploadProfile(result.getDouble("correct"), result.getDouble("wrongPlacement"), result.getDouble("insult"), result.getDouble("spam"), result.getDouble("violent"), result.getDouble("porn"), result.getDouble("upLoadContent"));
					//View profile settings
					viewProfile = new ViewProfile(result.getDouble("forum"), result.getDouble("theReporter"), result.getDouble("photoVideo"), result.getInt("viewMode"));
					//Complaint Profile
					complaintProfile = new ComplaintProfile(result.getDouble("wrongPlacementComplaint"), result.getDouble("insultComplaint"), result.getDouble("spamComplaint"), result.getDouble("violentComplaint"), result.getDouble("pornComplaint"));
					
					contextData = new ContextData(randomUpload);

					int random = 0;
					int numAgent = 0;
					if(result.getString("name").equals("Moderate")){
						random = 100;
						numAgent = 1;
					}
					if(result.getString("name").equals("Spammer")){
						random = 200;
						numAgent = 2;
					}
					if(result.getString("name").equals("Rude")){
						random = 300;
						numAgent = 3;
					}
					if(result.getString("name").equals("Troll")){
						random = 400;
						numAgent = 4;
					}
					if(result.getString("name").equals("Violent")){
						random = 500;
						numAgent = 5;
					}
					if(result.getString("name").equals("Pornographic")){
						random = 600;
						numAgent = 6;
					}
					randomUpload = new Random(random);
					Agent moderate = new Agent(numAgent,quantity.get(i),result.getString("name"),uploadProfile,viewProfile,complaintProfile, population);
					
					agente.add(moderate);
				}
			} catch (SQLException e) {
				e.printStackTrace();
				System.out.println("Error with SQL query. The format isn't correct, see the manual of Mysql");
			}
		}
	
		return agente;
		
	}

	/**
	 * Method to save new population to BBDD
	 * @param agente Arraylist of the agents
	 * @param popName Population name
	 */
	public void setNewAgentPopulation(ArrayList<Agent> agents, String popName) {
	
		ArrayList<Integer> agent_type_id= new ArrayList<Integer>();
		ArrayList<Integer> agent_id_profile = new ArrayList<Integer>();
		String stmt = null;
		int id_population = 0;
		
		try {
			
	        Statement statement = connection.createStatement();
	        stmt = "INSERT INTO agent_population (id_agent_population, populationName) VALUES (NULL,'"+popName+"')";
	        statement.executeUpdate(stmt);
	        
			for(int i = 0 ; i < agents.size() ; i++){
				stmt=
				"INSERT INTO agent_profile (`id_profile`, `name`, `correct`, `wrongPlacement`, `insult`, `spam`, `violent`, `porn`, `upLoadContent`, `forum`, `theReporter`, `photoVideo`, `viewMode`, `distributionMode`, `wrongPlacementComplaint`, `insultComplaint`, `spamComplaint`, `violentComplaint`, `pornComplaint`, `population`)" 
				+"VALUES (NULL, '"
				+agents.get(i).getName()+"', "
				+agents.get(i).getUp().getCorrect()+","
				+agents.get(i).getUp().getWrongPlacement()+","
				+agents.get(i).getUp().getInsult()+","
				+agents.get(i).getUp().getSpam()+", "
				+agents.get(i).getUp().getViolent()+", "
				+agents.get(i).getUp().getPorn()+","
				+agents.get(i).getUp().getUpLoadContent()+", "
				+agents.get(i).getVp().getForum()+", "
				+agents.get(i).getVp().getTheReporter()+","
				+agents.get(i).getVp().getPhotoVideo()+","
				+"0," //view mode
				+"0," // distribution mode
				+agents.get(i).getCp().getWrongPlacement()+","
				+agents.get(i).getCp().getInsult()+","
				+agents.get(i).getCp().getSpam()+","
				+agents.get(i).getCp().getViolent()+","
				+agents.get(i).getCp().getPorn()+",'"
				+popName+"');";
				System.out.println(stmt);
				statement.executeUpdate(stmt);
			}
			
			try{
				
				PreparedStatement statemnt;
				ResultSet result;
				
				statemnt = connection.prepareStatement("SELECT * FROM agent_population WHERE populationName = '"+popName+"'");
				result = statemnt.executeQuery();
				while(result.next()){
					id_population = result.getInt("id_agent_population");
				}
				
				statemnt = connection.prepareStatement("SELECT name, id_profile FROM agent_profile WHERE population = '"+popName+"'");
				result = statemnt.executeQuery();
				ArrayList<String> profileName = new ArrayList<String>();
				while(result.next()){
					profileName.add(result.getString("name"));
					agent_id_profile.add(result.getInt("id_profile"));
				}
				
				for(int i = 0 ; i < profileName.size() ; i++){	
					statemnt = connection.prepareStatement("SELECT id_agent_type FROM agent_type WHERE typeName = '"+profileName.get(i)+"'");
					result = statemnt.executeQuery();
					while(result.next()){
						agent_type_id.add(result.getInt("id_agent_type"));
					}
				}
				
			} catch (SQLException e) {
				e.printStackTrace();
				System.out.println("Error with SQL query. The format isn't correct, see the manual of Mysql");
			}
			
			for(int i = 0 ; i < agents.size() ; i++){
				stmt = "INSERT INTO population (id_agent_type, id_agent_profile, id_population, quantity) VALUES ("+agent_id_profile.get(i)+","+agent_id_profile.get(i)+","+id_population+","+agents.get(i).getQuantity()+")";
				statement.executeUpdate(stmt);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}




