 package social.context;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import repast.simphony.context.Context;
import repast.simphony.context.space.continuous.ContinuousSpaceFactory;
import repast.simphony.context.space.continuous.ContinuousSpaceFactoryFinder;
import repast.simphony.context.space.grid.GridFactory;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.RandomCartesianAdder;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.SimpleGridAdder;
import repast.simphony.space.grid.WrapAroundBorders;
import repast.simphony.ui.RSApplication;
import social.agents.Agent;
import social.agents.ModerateAgent;
import social.agents.PornographicAgent;
import social.agents.RudeAgent;
import social.agents.SpammerAgent;
import social.agents.TrollAgent;
import social.agents.ViolentAgent;
import social.agents.profile.ComplaintProfile;
import social.agents.profile.UploadProfile;
import social.agents.profile.ViewProfile;
import social.section.SectionForum;
import social.section.SectionPhotoVideo;
import social.section.SectionTheReporter;

/**
 * Context builder of the simulation
 * @author davidsanchezpinsach
 *
 */
public class VirtualCommunitiesContext implements ContextBuilder<Object> {

	static ContextData contextData; //Context data of the context
	static ContinuousSpace<Object> space; //Space of the context
	static Grid<Object> grid; //Grid of the context
	static Context<Object> context;
	
	private int row = 0;

	//Random and seed of the random numbers
	private Random random;
	private int seed = 100; 
		
	//Schedule parameters
	private double start = 1, interval = 1, priority = 0;
	
	public Context<Object> build(Context<Object> context) {
			
		VirtualCommunitiesContext.context = context;
		
		//Create a new context data
		random = new Random(seed);
		contextData = new ContextData(random);
		
		context.setId("VirtualCommunitiesContext");
		ContinuousSpaceFactory spaceFactory = ContinuousSpaceFactoryFinder.createContinuousSpaceFactory(null);
		space = spaceFactory.createContinuousSpace("space", context ,
				new RandomCartesianAdder <Object >(),
				new repast.simphony.space.continuous.WrapAroundBorders(), contextData.getNumColumns(), contextData.getNumRows());
				GridFactory gridFactory = GridFactoryFinder.createGridFactory(null); 
				grid = gridFactory.createGrid("grid", context,
						new GridBuilderParameters<Object>(new WrapAroundBorders(),
								new SimpleGridAdder<Object>(), false, contextData.getNumColumns(), contextData.getNumRows()));  //False determinate that we only allow one object by position
		
		//Put the context data in position 0,0
		context.add(contextData);
		space.moveTo(contextData, 0, 0);
		grid.moveTo(contextData, 0, 0);
		
		//Make the user panel.
		makeUserPanel();
		
		//Open the agent configuration window.
		ConfigureAgentWindow caw = new ConfigureAgentWindow(new javax.swing.JFrame(), true);	
		createAgentsFromConfigureWindows(caw.getAgente());

		return context;
	}
	
	/**
	 * Method to create sections of the simulation
	 */
	private void makeSections() {
		//Add the section title to display
		row = contextData.getNumRows()-1;
		//Section The reporter
		SectionTheReporter sectionTheReporter = new SectionTheReporter();
		context.add(sectionTheReporter);
		space.moveTo(sectionTheReporter, (contextData.getMaxCommentsTheReporter()+1)/2, row);
		grid.moveTo(sectionTheReporter, (contextData.getMaxCommentsTheReporter()+1)/2, row);
		//Section Forum
		SectionForum sectionForum = new SectionForum();
		context.add(sectionForum);
		space.moveTo(sectionForum, contextData.getMaxCommentsTheReporter()+1+(contextData.getMaxCommentsForum()+1)/2, row);
		grid.moveTo(sectionForum, contextData.getMaxCommentsTheReporter()+1+(contextData.getMaxCommentsForum()+1)/2, row);
		//Section Photo & Video
		SectionPhotoVideo sectionPhotoVideo = new SectionPhotoVideo();
		context.add(sectionPhotoVideo);
		space.moveTo(sectionPhotoVideo, contextData.getMaxCommentsTheReporter()+contextData.getMaxCommentsForum()+2+(contextData.getMaxCommentsPhotoVideo()+1)/2, row);
		grid.moveTo(sectionPhotoVideo, contextData.getMaxCommentsTheReporter()+contextData.getMaxCommentsForum()+2+(contextData.getMaxCommentsPhotoVideo()+1)/2, row);
	
	}

	/**
	 * Method to create agents from configure windows
	 * @param agente ArrayList of agents
	 */
	public void createAgentsFromConfigureWindows(ArrayList<Agent> agents){
		row = 0;
		makeSections();
		for(int i = 0 ; i < agents.size() ; i++){
			for(int j = 0 ; j < agents.get(i).getQuantity() ; j++){				
				row = contextData.getNumRows() - (2 * contextData.getNumAgents()) - 2;			
				
				UploadProfile up = new UploadProfile(
						agents.get(i).getUp().getCorrect(),
						agents.get(i).getUp().getWrongPlacement(),
						agents.get(i).getUp().getInsult(),
						agents.get(i).getUp().getSpam(),
						agents.get(i).getUp().getViolent(),
						agents.get(i).getUp().getPorn(),
						agents.get(i).getUp().getUpLoadContent());
				up.setTotalComments(contextData.getTotalComments());
				//up.setTotalComments(1000);
				up.setMaxCommentsTheReporter(contextData.getMaxCommentsTheReporter());
				up.setMaxCommentsForum(contextData.getMaxCommentsForum());
				up.setMaxCommentsPhotoVideo(contextData.getMaxCommentsPhotoVideo());
					
				ViewProfile vp = new ViewProfile(
						agents.get(i).getVp().getForum(),
						agents.get(i).getVp().getTheReporter(),
						agents.get(i).getVp().getPhotoVideo(),
						agents.get(i).getVp().getViewMode());

				ComplaintProfile cp = new ComplaintProfile(
						agents.get(i).getCp().getWrongPlacement(),
						agents.get(i).getCp().getInsult(),
						agents.get(i).getCp().getSpam(),
						agents.get(i).getCp().getViolent(),
						agents.get(i).getCp().getPorn());

				switch(agents.get(i).getType()){
					case 1:
						moderateAgentCreation(agents, i, up, vp, cp);
						break;
					case 2:
						spammerAgentCreation(agents, i, up, vp, cp);
						break;
					case 3:
						rudeAgentCreation(agents, i, up, vp, cp);
						break;
					case 4:
						trollAgentCreation(agents, i, up, vp, cp);
						break;
					case 5:
						violentAgentCreation(agents, i, up, vp, cp);
						break;
					case 6:
						pornoAgentCreation(agents, i, up, vp, cp);
						break;
						
					default:
						break;
				}
				contextData.setNumAgents(contextData.getNumAgents()+1);
			}
		}
	}

	/* Methods to create the different agents*/
	private void violentAgentCreation(ArrayList<Agent> agente, int i, UploadProfile up, ViewProfile vp, ComplaintProfile cp) {
		ViolentAgent violentAgent = new ViolentAgent(space, grid, row, up, vp, cp);
		context.add(violentAgent);
		space.moveTo(violentAgent, 0, row);
		grid.moveTo(violentAgent, 0, row);
		configureScheduleAgent(violentAgent, start, interval, priority);		
	}

	private void trollAgentCreation(ArrayList<Agent> agente, int i, UploadProfile up, ViewProfile vp, ComplaintProfile cp) {
		TrollAgent trollAgent = new TrollAgent(space, grid, row, up, vp, cp);
		context.add(trollAgent);
		space.moveTo(trollAgent, 0, row);
		grid.moveTo(trollAgent, 0, row);
		configureScheduleAgent(trollAgent, start, interval, priority);		
	}

	private void spammerAgentCreation(ArrayList<Agent> agente, int i, UploadProfile up, ViewProfile vp, ComplaintProfile cp) {
		SpammerAgent spammerAgent = new SpammerAgent(space, grid, row, up, vp, cp);
		context.add(spammerAgent);
		space.moveTo(spammerAgent, 0, row);
		grid.moveTo(spammerAgent, 0, row);
		priority += 1;
		configureScheduleAgent(spammerAgent, start, interval, priority);		
	}

	private void rudeAgentCreation(ArrayList<Agent> agente, int i, UploadProfile up, ViewProfile vp, ComplaintProfile cp) {
		RudeAgent rudeAgent = new RudeAgent(space, grid, row, up, vp, cp);
		context.add(rudeAgent);
		space.moveTo(rudeAgent, 0, row);
		grid.moveTo(rudeAgent, 0, row);
		priority += 1;
		configureScheduleAgent(rudeAgent, start, interval, priority);		
	}

	private void pornoAgentCreation(ArrayList<Agent> agente, int i, UploadProfile up, ViewProfile vp, ComplaintProfile cp) {
		PornographicAgent pornographicAgent = new PornographicAgent(space, grid, row, up, vp, cp);
		context.add(pornographicAgent);
		space.moveTo(pornographicAgent, 0, row);
		grid.moveTo(pornographicAgent, 0, row);
		configureScheduleAgent(pornographicAgent, start, interval, priority);		
	}

	private void moderateAgentCreation(ArrayList<Agent> agente, int i, UploadProfile up, ViewProfile vp, ComplaintProfile cp) {
		ModerateAgent moderateAgent = new ModerateAgent(space, grid, row, up, vp, cp);
		context.add(moderateAgent);
		space.moveTo(moderateAgent, 0, row);
		grid.moveTo(moderateAgent, 0, row);
		priority += 1;
		configureScheduleAgent(moderateAgent, start, interval, priority);		
	}

	/**
	 * Method to create schedule for the agent
	 * @param agent Agent
	 * @param start Start of tick action of the agent
	 * @param interval Interval of tick action of the agent
	 * @param priority Piority of the schedule of this agent
	 */
	private void configureScheduleAgent(Agent agent, double start, double interval, double priority){
		ScheduleParameters scheduleParams;
		ISchedule schedule;
		schedule = RunEnvironment.getInstance().getCurrentSchedule();
		//Create scheduler for upload content
		scheduleParams = ScheduleParameters.createRepeating(start, interval, priority);
		schedule.schedule(scheduleParams, agent, "upLoadContent");
		
		//Create scheduler for view and complaint content
		scheduleParams = ScheduleParameters.createRepeating(start, interval, priority+contextData.getMaxAgents());
		schedule.schedule(scheduleParams, agent, "viewAndComplaintContent");
	}
	
	/**
	 * Method to put button of the configure windows into user panel
	 */
	private void makeUserPanel(){
		JPanel panel = new JPanel();
		
		panel.setBorder(new TitledBorder("Setup of simulation"));
	    panel.setLayout(new GridLayout(6,2,15,0));//6 rows, 2 cols
	    
	    JButton button = new JButton("Setup agents");
	    JLabel label = new JLabel("Configure your agent"); 
	    
	    panel.add(label);     
	    panel.add(button);
	    
	    //Add action listener to button
	    button.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e){
	        	ConfigureAgentWindow caw = new ConfigureAgentWindow(new javax.swing.JFrame(), true);	
	    		createAgentsFromConfigureWindows(caw.getAgente());
	        }
	    });    
	    RSApplication.getRSApplicationInstance().removeCustomUserPanel();
		RSApplication.getRSApplicationInstance().addCustomUserPanel(panel);	
	}
}
